#ifndef MVOPSTEST_H
#define MVOPSTEST_H

#include <iostream>
#include <stdlib.h>
#include "Mat2d.h"
#include "Vec.h"
#include "MVoperators.h"

bool testMatOps() {
   const int m = 300, n = 200;
   Mat2d M1(m, n), M2(m, n), M3(m, n);

   M1 = 1;
   M2 = 0;
   M2 += M1 * 2; // M2 = 2
   M2 /= 4; // M2 = .5
   M2 *= 2; // M2 = 1
   M2 -= 0; // M2 = 1
   if(-1 + M2 + 1 != 1 - M2 - 1 + 2)
      return false;
   if(4 * M2 * .25 != 4 / M2 / 4)
      return false;
   if(M1 * M2 != M1 / M2)
      return false;
   // M3 = rand():
   for(int i = 0; i < m; ++i) {
      for(int j = 0; j < n; ++j) {
          M3(i, j) = rand();
      }
   }
   Mat2d M4(M3 * M3);
   for(int i = 0; i < m; ++i) {
      for(int j = 0; j < n; ++j) {
         if(M4(i, j) != M3(i, j) * M3(i, j))
            return false;
      }
   }

   return true;
}

bool testVecOps() {
   const int n = 200;
   Vec V1(n), V2(n), V3(n);

   V1 = 1;
   V2 = 0;
   V2 += V1 * 2; // V2 = 2
   V2 /= 4; // V2 = .5
   V2 *= 2; // V2 = 1
   V2 -= 0; // V2 = 1
   if(-1 + V2 + 1 != 1 - V2 - 1 + 2)
      return false;
   if(4 * V2 * .25 != 4 / V2 / 4)
      return false;
   if(V1 * V2 != V1 / V2)
      return false;
   // V3 = rand():
   for(int i = 0; i < n; ++i) {
       V3(i) = rand();
   }
   Vec V4(V3 * V3);
   for(int i = 0; i < n; ++i) {
      if(V4(i) != V3(i) * V3(i))
         return false;
   }

   return true;
}

bool testMatVecProduct() {
   const int m = 200, n = 500;
   Mat2d M(m, n);
   Vec V(n);
   M = 1;
   V = 1;
   Vec Vtest(m);
   Vtest = n;
   if(M * V != Vtest)
      return false;
   return true;
}

bool testMatVecOps() {
   bool result = true;
   if(!testMatOps()) {
      std::cerr << "MatOps test failed" << std::endl;
      result = false;
   }
   if(!testVecOps()) {
      std::cerr << "VecOps test failed" << std::endl;
      result = false;
   }
   if(!testMatVecProduct()) {
      std::cerr << "MatVecProduct test failed" << std::endl;
      result = false;
   }

   return result;
}

#endif
